#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform sampler2D TexSampler;

layout( std140) uniform UToneMappingInfo
{
	vec4 ExpGamma;
} ToneMappingInfo;

vec3 UnpackNormalSpheremap(vec2 normal)
{
	vec4 nn = vec4(normal * 2.0 - 1.0, 1., -1.);
	float l = dot(nn.xyz, -nn.xyw);
	nn.z = l;
	nn.xy *= sqrt(l);
	return nn.xyz + vec3(0.5, 0.5, 0.);
}

void FragmentMain(in SFragmentInput fInput, out SFragmentOutput fOutput)
{



	vec2 uv = fInput.fragTexCoord;


	fOutput.outColor =  texture(TexSampler, uv) ;
	float exposure =  ToneMappingInfo.ExpGamma .x;
	vec3 gammaInv = 1. /  ToneMappingInfo.ExpGamma .yyy;

	vec3 colorMapped = 1.f - exp(- exposure * fOutput.outColor.rgb);
	vec3 colorCorrected = pow(colorMapped, gammaInv);

	fOutput.outColor.rgb = colorCorrected;

}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
