#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) in vec3 _vin_Position;
layout(location = 1) in vec2 _vin_Size;
layout(location = 2) in float _vin_Rotation;
layout(location = 3) in vec3 _vin_Axis0;
layout(location = 4) in vec3 _vin_Axis1;
layout(location = 5) in uint _vin_Enabled;
layout(location = 6) in float _vin_IsSelected;

struct SVertexInput
{
	vec3 Position;
	vec2 Size;
	float Rotation;
	vec3 Axis0;
	vec3 Axis1;
	uint Enabled;
	float IsSelected;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec4 _vout_geomColor;
layout(location = 1) smooth out vec2 _vout_geomSize;
layout(location = 2) smooth out float _vout_geomRotation;
layout(location = 3) smooth out vec3 _vout_geomAxis0;
layout(location = 4) smooth out vec3 _vout_geomAxis1;
layout(location = 5) smooth out float _vout_geomIsSelected;

struct SVertexOutput
{
	vec4 geomColor;
	vec2 geomSize;
	float geomRotation;
	vec3 geomAxis0;
	vec3 geomAxis1;
	float geomIsSelected;
	vec4 VertexPosition;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 InvView_FrameRhYUp;
	mat4 LHZToUser;
	vec4 ViewPosition;
	vec4 ViewDirection;
	vec4 DirLightPosition;
	vec4 DirLightColor;
	vec4 AmbientLightColor;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	float Handedness;
} SceneInfo;



void VertexMain( in SVertexInput  vInput,  out SVertexOutput  vOutput  )
{
	vOutput.VertexPosition = vec4(vInput.Position, 0);
	vOutput.geomSize = vInput.Size;


	vOutput.geomRotation = vInput.Rotation;



	if (vInput.Enabled == 0u)
		vOutput.geomSize *= 0.f;



	vOutput.geomAxis0 = vInput.Axis0;



	vOutput.geomAxis1 = vInput.Axis1;





	vOutput.geomIsSelected = vInput.IsSelected;
	vOutput.geomColor = vInput.IsSelected == 2.0f ? vec4(0.8, 0.0, 0.0, 1.0) : vec4(0.1, 0.1, 0.1, 1.0);
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.Position = _vin_Position;
	vInput.Size = _vin_Size;
	vInput.Rotation = _vin_Rotation;
	vInput.Axis0 = _vin_Axis0;
	vInput.Axis1 = _vin_Axis1;
	vInput.Enabled = _vin_Enabled;
	vInput.IsSelected = _vin_IsSelected;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.geomColor = vec4(0, 0, 0, 0);
	vOutput.geomSize = vec2(0, 0);
	vOutput.geomRotation = float(0);
	vOutput.geomAxis0 = vec3(0, 0, 0);
	vOutput.geomAxis1 = vec3(0, 0, 0);
	vOutput.geomIsSelected = float(0);
	VertexMain(vInput, vOutput);
	_vout_geomColor = vOutput.geomColor;
	_vout_geomSize = vOutput.geomSize;
	_vout_geomRotation = vOutput.geomRotation;
	_vout_geomAxis0 = vOutput.geomAxis0;
	_vout_geomAxis1 = vOutput.geomAxis1;
	_vout_geomIsSelected = vOutput.geomIsSelected;
	gl_Position = vOutput.VertexPosition;
}
