#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) in vec3 _vin_Position;
layout(location = 1) in float _vin_Size;
layout(location = 2) in float _vin_Rotation;
layout(location = 3) in uint _vin_Enabled;
layout(location = 4) in vec4 _vin_Color;

struct SVertexInput
{
	vec3 Position;
	float Size;
	float Rotation;
	uint Enabled;
	vec4 Color;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec4 _vout_geomColor;
layout(location = 1) smooth out float _vout_geomSize;
layout(location = 2) smooth out float _vout_geomRotation;

struct SVertexOutput
{
	vec4 geomColor;
	float geomSize;
	float geomRotation;
	vec4 VertexPosition;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 InvView_FrameRhYUp;
	mat4 LHZToUser;
	vec4 ViewPosition;
	vec4 ViewDirection;
	vec4 DirLightPosition;
	vec4 DirLightColor;
	vec4 AmbientLightColor;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	float Handedness;
} SceneInfo;



void VertexMain( in SVertexInput  vInput,  out SVertexOutput  vOutput  )
{
	vOutput.VertexPosition = vec4(vInput.Position, 0);
	vOutput.geomSize = vInput.Size;


	vOutput.geomRotation = vInput.Rotation;



	if (vInput.Enabled == 0u)
		vOutput.geomSize *= 0.f;
	vOutput.geomColor = vInput.Color;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.Position = _vin_Position;
	vInput.Size = _vin_Size;
	vInput.Rotation = _vin_Rotation;
	vInput.Enabled = _vin_Enabled;
	vInput.Color = _vin_Color;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.geomColor = vec4(0, 0, 0, 0);
	vOutput.geomSize = float(0);
	vOutput.geomRotation = float(0);
	VertexMain(vInput, vOutput);
	_vout_geomColor = vOutput.geomColor;
	_vout_geomSize = vOutput.geomSize;
	_vout_geomRotation = vOutput.geomRotation;
	gl_Position = vOutput.VertexPosition;
}
