#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) in vec3 _vin_Position;
layout(location = 1) in vec3 _vin_Normal;
layout(location = 2) in vec2 _vin_TexCoord;

struct SVertexInput
{
	vec3 Position;
	vec3 Normal;
	vec2 TexCoord;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec3 _vout_fragNormal;
layout(location = 1) smooth out vec2 _vout_fragTexCoord;
layout(location = 2) smooth out vec4 _vout_fragWorldPosition;

struct SVertexOutput
{
	vec3 fragNormal;
	vec2 fragTexCoord;
	vec4 fragWorldPosition;
	vec4 VertexPosition;
};


layout( std140) uniform UMeshVertexConstant
{
	mat4 ModelViewProj;
	mat4 NormalMatrix;
} MeshVertexConstant;



void VertexMain(in SVertexInput vInput, out SVertexOutput vOutput)
{
    vOutput.VertexPosition =  (( MeshVertexConstant.ModelViewProj ) * (vec4(vInput.Position, 1.0))) ;
    vOutput.fragNormal =  (( MeshVertexConstant.NormalMatrix ) * (vec4(vInput.Normal, 0))) .xyz;
    vOutput.fragTexCoord = vInput.TexCoord;
	vOutput.fragWorldPosition = vOutput.VertexPosition;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.Position = _vin_Position;
	vInput.Normal = _vin_Normal;
	vInput.TexCoord = _vin_TexCoord;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.fragNormal = vec3(0, 0, 0);
	vOutput.fragTexCoord = vec2(0, 0);
	vOutput.fragWorldPosition = vec4(0, 0, 0, 0);
	VertexMain(vInput, vOutput);
	_vout_fragNormal = vOutput.fragNormal;
	_vout_fragTexCoord = vOutput.fragTexCoord;
	_vout_fragWorldPosition = vOutput.fragWorldPosition;
	gl_Position = vOutput.VertexPosition;
}
